/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <s3c2410.h>
#include <exports.h>
#include "LATLOG9A.H"

#define M5D(n) ((n) & 0x1fffff)	// To get lower 21bits
#define U32 unsigned int

#if 0
#define rLCDCON1    (*(volatile unsigned *)0x4d000000) //LCD control 1
#define rLCDCON2    (*(volatile unsigned *)0x4d000004) //LCD control 2
#define rLCDCON3    (*(volatile unsigned *)0x4d000008) //LCD control 3
#define rLCDCON4    (*(volatile unsigned *)0x4d00000c) //LCD control 4
#define rLCDCON5    (*(volatile unsigned *)0x4d000010) //LCD control 5
#define rLCDSADDR1  (*(volatile unsigned *)0x4d000014) //STN/TFT Frame buffer start address 1
#define rLCDSADDR2  (*(volatile unsigned *)0x4d000018) //STN/TFT Frame buffer start address 2
#define rLCDINTPND  (*(volatile unsigned *)0x4d000054) //LCD Interrupt pending
#define rLCDSRCPND  (*(volatile unsigned *)0x4d000058) //LCD Interrupt source
#define rLCDINTMSK  (*(volatile unsigned *)0x4d00005c) //LCD Interrupt mask
#define rLPCSEL     (*(volatile unsigned *)0x4d000060) //LPC3600 Control
#define rTPAL       (*(volatile unsigned *)0x4d000050) //TFT Temporary palette
#endif

#define LCDFRAMEBUFFER 	0x30800000 //_NONCACHE_STARTADDRESS 
#define LCD_XSIZE 	(320)
#define LCD_YSIZE 	(240)
#define SCR_XSIZE 	(LCD_XSIZE*1)   //for virtual screen  
#define SCR_YSIZE	(LCD_YSIZE*1)

void FB_Init(int vram_addr, int width, int height, int virtual_width, int virtual_height);
void FB_Putpixel(int x, int y, int color);
void FB_DrawBMP(int x, int y, const unsigned char *fp);

int fb_screen (int argc, char *argv[])
{
	int i;

	/* Print the ABI version */
	app_startup(argv);
	printf ("Example expects ABI version %d\n", XF_VERSION);
	printf ("Actual U-Boot ABI version %d\n", (int)get_version());

	printf ("Initializing Frame Buffer\n");
        FB_Init(LCDFRAMEBUFFER,LCD_XSIZE,LCD_YSIZE,SCR_XSIZE,SCR_YSIZE);
	printf ("Drawing Bitmap to Frame Buffer\n");
        for(i=0;i<100000;i++);
        FB_DrawBMP(0, 0,latlog9a);
	printf ("Bitmap Drawn to Frame Buffer\n");
	printf ("Hit any key to exit ... ");
	while (!tstc())
		;
	/* consume input */
	(void) getc();

	printf ("\n\n");
	return (0);
}

void FB_Init(int vram_addr, int width, int height, int virtual_width, int virtual_height)
{
     S3C24X0_GPIO * const gpio = S3C24X0_GetBase_GPIO();
     S3C24X0_LCD * const lcd = S3C24X0_GetBase_LCD();

//     gpio->GPCUP=0xFFFFFFFF; 			// Disable Pull-up register
//     gpio->GPCCON=0xAAAAAAAA; 			// Initialize VD[7:0],LCDVF[2:0],VM,VFRAME,VLINE,VCLK,LEND 
     lcd->LCDCON2=((3-1)<<24)|((height-1)<<14)|((6-1)<<6)|((4-1)<<0);
     gpio->GPCCON &= ~(0x3<<8);

     gpio->GPCCON |= (0x1<<8);	  //DE Ioport
     gpio->GPCDAT|= (1<<4);	 	  //DE High
	 
//     gpio->GPDUP=0xFFFFFFFF; 			// Disable Pull-up register
//     gpio->GPDCON=0xAAAAAAAA; 			// Initialize VD[23:8]	

	 //GPG4 is setted as LCD_PWREN
	 
     gpio->GPGUP|=(1<<4);				// Pull-up disable
     gpio->GPGCON|=(3<<8);				// GPG4=LCD_PWREN
     
     lcd->LCDCON1=(5<<8)|(0<<7)|(3<<5)|(12<<1)|0;
     // 16bpp TFT, ENVID=off
     lcd->LCDCON2=((14-1)<<24)|((height-1)<<14)|((23-1)<<6)|((4-1)<<0);

     lcd->LCDCON3=((58-1)<<19)|((width-1)<<8)|((30-1)<<0);
     lcd->LCDCON4=(13<<8)|((10-1)<<0);
     lcd->LCDCON5=(1<<11)|(1<<10)|(1<<9)|(1<<8)|(1<<7)|(1<<0);	// FRM5:6:5,HSYNC and VSYNC are inverted
     
     // BPP24BL:x,FRM565:x,INVVCLK:x,INVVLINE:x,INVVFRAME:x,INVVD:x,
     // INVVDEN:x,INVPWREN:x,INVLEND:x,PWREN:x,ENLEND:x,BSWP:x,HWSWP:x
     lcd->LCDSADDR1=(((U32)vram_addr>>22)<<21 )|M5D((U32)vram_addr>>1);
     lcd->LCDSADDR2=M5D( ((U32)vram_addr+(virtual_width*height*2))>>1 );
     lcd->LCDSADDR3=(((virtual_width-width)/1)<<11)|(width/1);
    
     lcd->LCDINTMSK|=(3); // MASK LCD Sub Interrupt
     lcd->LPCSEL&=(~7); // Disable LPC3600
     lcd->TPAL=0; // Disable Temp Palette
     //Enable LCD POWER ENABLE Function
     lcd->LCDCON5|=(1<<3);   			// PWREN
 //    rLCDCON5&=~(1<<5);   			// INVPWREN
     
     lcd->LCDCON1|=1; // ENVID=ON
     		
}	

void FB_Putpixel(int x, int y, int color)
{
     unsigned int addr;	       	
     
     addr=LCDFRAMEBUFFER+(x+y*SCR_XSIZE)*2;
     *(volatile short *)(addr)= (short) color;    
} 

void FB_DrawBMP(int x, int y, const unsigned char *fp)
{
     int xx=0, yy=0;	
     unsigned int tmp;
     unsigned char tmpR, tmpG, tmpB;
     
     unsigned short bfType;
     unsigned int bfSize;
     unsigned int bfOffbits;
     unsigned int biWidth, biWidth2;
     unsigned int biHeight;     
	
     bfType=*(unsigned short *)(fp+0);
     bfSize=*(unsigned short *)(fp+2);
     tmp=*(unsigned short *)(fp+4);
     bfSize=(tmp<<16)+bfSize;
     bfOffbits=*(unsigned short *)(fp+10);
     biWidth=*(unsigned short *)(fp+18);    
     biHeight=*(unsigned short *)(fp+22);          
     
     biWidth2=(bfSize-bfOffbits)/biHeight;	
   
     
     for(yy=0;yy<biHeight;yy++)
     {
         for(xx=0;xx<biWidth2/3;xx++)
         {
             tmpB=*(unsigned char *)(fp+bfOffbits+3*xx+(biHeight-yy-1)*biWidth2+0);
             tmpG=*(unsigned char *)(fp+bfOffbits+3*xx+(biHeight-yy-1)*biWidth2+1);
             tmpR=*(unsigned char *)(fp+bfOffbits+3*xx+(biHeight-yy-1)*biWidth2+2);   	
	     tmpR>>=3;
	     tmpG>>=3;
	     tmpB>>=3;
	             
	     if(xx<biWidth2)
	         FB_Putpixel(x+xx,y+yy,(tmpR<<11)+(tmpG<<6)+(tmpB<<0));
         } 
     } 
}
