/* hdd_ln2410sbc.c */
/* Hard Disk Test program for LN2410SBC version 1.6 */

#include <common.h>
#include <s3c2410.h>
#include <s3c24x0.h>
#include <exports.h>

#define U32 unsigned int
#define U16 unsigned short int
#define U8 unsigned char

#define CMD_DATA 	   0x0
#define CMD_ERR  	   0x1
#define CMD_FEATURE  0x1
#define CMD_SECCNT   0x2
#define CMD_SECNUM   0x3
#define CMD_CYLL     0x4
#define CMD_CYLH     0x5
#define CMD_DEVICE   0x6
#define CMD_COMMAND  0x7
#define CMD_STATUS   0x7

#define CTL_ASTATUS  0x06
#define CTL_CONTROL  0x06
#define CTL_ADDRESS  0x07

#define BIT_BSY      0x80
#define BIT_DRDY     0x40
#define BIT_DRQ      0x08
#define BIT_ERR      0x01


#define HDD_DATA          0x11000000
#define HDD_CMD           0x11000000
#define HDD_CTL		  0x15000000

//#define _ISR_STARTADDRESS 0x30000000
volatile U16*   IDECMD;
volatile U16*  IDEDATA;
volatile U16*   IDECTL;
volatile U8*   IDEATTRIB;
volatile U8*   IDECONFREG;

void InitHDD(void);
void ViewBuff(unsigned char *buff);
void IDE_ErrorMessage(U16 stat);
void Delay400ns(void);
void IDE_WaitDeviceReady(void);
U16 IDE_WaitReady(void);
U16 IDE_WaitDeviceRequest(void);
void IDE_Reset(void);
int IDE_IsCompleteTransfer(void);
void IDE_DeviceSelect(void);
void IDE_IdentifyDevice(void);
void IDE_ReadSector(void);
void ISR_EINT12(void);


int cf_ln2410hdd (int argc, char *argv[])
{

   int i;
   S3C24X0_GPIO * const gpio = S3C24X0_GetBase_GPIO();;

   app_startup(argv);
   printf ("Example expects ABI version %d\n", XF_VERSION);
   printf ("Actual U-Boot ABI version %d\n", (int)get_version());
   printf("\n HDD Storage Test \n");
   InitHDD();
   printf(" Initialize HDD \n");

// LED
   gpio->GPBCON=(gpio->GPBCON&~(0x3<<2))|(1<<2);
   gpio->GPBDAT=(gpio->GPBDAT|(0x1<<1));   // On

/*	
   pISR_EINT1 = (unsigned int)ISR_EINT1;
   rEXTINT0 = (rEXTINT0&~(0xf<<4)) | (0x2)<<4;
   gpio->GPFCON  = (gpio->GPFCON&~(0x3<<2)) | (0x2)<<2;
   ClearPending(BIT_EINT1);
   rINTMSK &= ~BIT_EINT1;
*/

   IDE_Reset();
   printf("Reset HDD \n");
   IDE_IdentifyDevice();
   printf ("Hit any key to exit ... ");
   i=0;
   while(!tstc())
   {
      if(i > 100000)
      {
        gpio->GPBDAT^=(1<<5);
        i=0;
      }
      i++;
   }
   (void) getc();
   printf ("\n\n");
   return (0);   
}

void InitHDD(void)
{
   S3C24X0_GPIO * const gpio = S3C24X0_GetBase_GPIO();
   gpio->GPBCON &= ~((0x3)<<14);
   IDECMD  = (volatile U16*)HDD_CMD;
   IDECTL  = (volatile U16*)HDD_CTL;
   IDEDATA = (volatile U16*)HDD_DATA;
 
}



void ViewBuff(unsigned char *buff)
{
	int i,j;
	unsigned char ch;
	
	printf("\n\r******************************************\n\r");
	for(i=0;i<512;i+=16)
	{
		for(j=0;j<16;j++)
		{
			printf("%02X ",buff[i+j]);
		}
		for(j=0;j<16;j++)
		{
			ch = buff[i+j];
			if( ch >= 128 ) ch -= 128;
			if( ch == '\n' || ch == '\n' )
				putc(' ');
			else
				putc(ch);
		}
		printf("\n\r");
	}
	printf("\n\r");
}


void Delay400ns(void)
{
  int i;
  for(i=0;i<10;i++);
} 

char* ErrString[] =
{
	"Address mark not found",
	"Track0 not found",
	"Aborted command",
	"Media change requested",
	"ID not found",
	"Media change",
	"Uncorrectable data error",
	"Bad block detected"
};

void IDE_ErrorMessage(U16 stat)
{
	U16 mask = 0x01;
	int i=0;
	
	for( i=0; i<8; i++, mask<<=1 )
		if( mask & stat ) printf("--> %s\n\r",ErrString[i]);
}

void IDE_WaitDeviceReady(void)
{
	U16 result, i;

	printf("Wait..");
	do
	{
		result=IDECMD[CMD_STATUS];
		//Delay400ns();
		for(i=0;i<10;i++);
	}while( result & (BIT_BSY|BIT_DRQ));
	printf("\rReady..\n\r");
}

U16 IDE_WaitReady(void)
{
	U16 result, i;

	do
	{
		result=IDECMD[CMD_STATUS];
		//Delay400ns();
		for(i=0;i<10;i++);
	} while( result & BIT_BSY);

	return result;
}

U16 IDE_WaitDeviceRequest(void)
{
	U16 result, i;

	printf("Wait DRQ..");
	
	if( IDE_WaitReady() & BIT_ERR )
	{
		result =  IDECMD[CMD_ERR]&0xff;
		printf("Error:%x\n\r",result);
		IDE_ErrorMessage(result);
		return 0;
	}
	printf("\n\rpass..\n\r");
	
	do
	{
		result=IDECMD[CMD_STATUS];
		for(i=0;i<10;i++);
	} while( (result&(BIT_BSY|BIT_DRQ)) != BIT_DRQ );

	printf("\rReady DRQ..\n\r");
	return 1;
}

void IDE_Reset(void)
{
	IDECMD[CMD_COMMAND]=0x08;
	IDE_WaitReady();
}

int IDE_IsCompleteTransfer(void)
{
	return (IDECTL[CTL_ASTATUS]&BIT_DRQ)==0? 1: 0;
}

void IDE_DeviceSelect(void)
{
	IDE_WaitDeviceReady();
	IDECMD[CMD_DEVICE] = 0x0; // reset command
	Delay400ns();
	IDE_WaitDeviceReady();
}

//Identify IDE Device
void IDE_IdentifyDevice(void)
{
	U16 buff[256];
	U16 result;
	U8  *cbuff;
	int i;
	
//	memset(buff,0,512);
        for(i=0;i<256;i++) buff[i] = 0;

	IDE_DeviceSelect();

	IDECMD[CMD_COMMAND] = 0xec;   	// Identify command
	for(i=0;i<10;i++); 				// Delay400ns();
	result = IDECTL[CTL_ASTATUS]; 	// ignore the result
	IDE_WaitDeviceRequest();

	i = 0;
	while( !IDE_IsCompleteTransfer() )
	{
             if(i<256)
             {
		buff[i] = *IDEDATA;   // HDD HCS0 Write
		i++;
             }
             else
             {
                break;
             }
	}
	printf("Number of data points %d \n",i);
	result = IDECTL[CTL_ASTATUS]; // ignore the result
	result = IDECMD[CMD_STATUS];  // ignore the result

	printf("\n");
	ViewBuff((unsigned char*)buff);

	printf("Serial Number:");
	cbuff = (U8*)&buff[10];
	for(i=0;i<20;i+=2)
	{
		putc(cbuff[i+1]);
		putc(cbuff[i]);
	}
	printf("\n\r");
	
	printf("Firmware Version:");
	cbuff = (U8*)&buff[23];
	for(i=0;i<8;i+=2)
	{
		putc(cbuff[i+1]);
		putc(cbuff[i]);
	}
	printf("\n\r");

	printf("Model Number:");
	cbuff = (U8*)&buff[27];
	for(i=0;i<40;i+=2)
	{
		putc(cbuff[i+1]);
		putc(cbuff[i]);
	}
	printf("\n\r");

	printf("Configuration: %x\n\r", buff[0]);
	printf("Num. of logical Cylinders    : %d\n\r", buff[1]);
	printf("Num. of logical Heads        : %d\n\r", buff[3]);
	printf("Num. of logical Sectors/Track: %d\n\r", buff[6]);

	printf("Current\n\r");
	printf("Num. of logical Cylinders    : %d\n\r", buff[54]);
	printf("Num. of logical Heads        : %d\n\r", buff[55]);
	printf("Num. of logical Sectors/Track: %d\n\r", buff[56]);
	i = buff[58]<<16 | buff[57];
	printf("Current capacity in Sectors  : %d\n\r", i);
	i = buff[61]<<16 | buff[60];
	printf("Total num. of available LBA  : %d\n\r", i);
}


//Ready sector
void IDE_ReadSector(void)
{
	U16 buff[256];
	U16 result;
	int i;
	
	for(i=0;i<256;i++) buff[i] = 0;

	IDE_DeviceSelect();

	IDECMD[CMD_SECCNT] = 0x1;
	IDECMD[CMD_SECNUM] = 0x1;
	IDECMD[CMD_CYLL]   = 0x0;
	IDECMD[CMD_CYLH]   = 0x0;
	IDECMD[CMD_DEVICE] = 0x0;
	
	IDECMD[CMD_COMMAND] = 0x20;   // read command
	
	Delay400ns();
	result = IDECTL[CTL_ASTATUS]; // ignore the result
	IDE_WaitDeviceRequest();

	i = 0;
	while( !IDE_IsCompleteTransfer() )
	{
		buff[i] = *IDEDATA;   // HDD HCS0 Write
		i++;
	}
	
	result = IDECTL[CTL_ASTATUS]; // ignore the result
	result = IDECMD[CMD_STATUS]; // ignore the result

	printf("\n\r");
	ViewBuff((unsigned char*)buff);
}

void ISR_EINT12(void)
{
	printf("Interrupt!!!!\n\r");
	ClearPending(BIT_EINT1);
}


